@@ -5,12 +5,7 @@ import inspect
 import json
 import textwrap
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Union
-
-from hera.host_config import get_global_task_image
-
-if TYPE_CHECKING:
-    from hera import DAG
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union, cast
 
 from argo_workflows.models import (
     Container,
@@ -30,8 +25,10 @@ from argo_workflows.models import VolumeMount
 import hera
 from hera.affinity import Affinity
 from hera.artifact import Artifact
+from hera.dag import DAG
 from hera.env import Env
 from hera.env_from import BaseEnvFrom
+from hera.global_config import GlobalConfig
 from hera.image import ImagePullPolicy
 from hera.io import IO
 from hera.memoize import Memoize
@@ -43,6 +40,7 @@ from hera.resources import Resources
 from hera.retry_strategy import RetryStrategy
 from hera.security_context import TaskSecurityContext
 from hera.sequence import Sequence
+from hera.sidecar import Sidecar
 from hera.template_ref import TemplateRef
 from hera.toleration import Toleration
 from hera.validators import validate_name
@@ -93,13 +91,19 @@ class Task(IO):
     with_sequence: Optional[Sequence] = None
         Sequence is similar to `with_param` in that it generates a range of objects. See `hera.sequence.Sequence` or
         https://argoproj.github.io/argo-workflows/fields/#sequence.
-    inputs: Optional[List[Union[Parameter, Artifact]]] = None
-        `Input` or `Parameter` objects that hold parameter inputs. Note that while `InputFrom` is an accepted input
-        parameter it cannot be used in conjunction with other types of inputs because of the dynamic aspect of the task
-        creation process when provided with an `InputFrom`.
+    inputs: Optional[
+            Union[
+                List[Union[Parameter, Artifact]],
+                List[Union[Parameter, Artifact, Dict[str, Any]]],
+                Dict[str, Any],
+            ]
+    ] = None,
+        `Input` or `Parameter` objects that hold parameter inputs. When a dictionary is specified all the key/value
+        pairs will be transformed into `Parameter`s. The `key` will be the `name` field of the `Parameter` while the
+        `value` will be the `value` field of the `Parameter.
     outputs: Optional[List[Union[Parameter, Artifact]]] = None
         `Output` objects that dictate the outputs of the task.
-    dag: Optional["DAG"] = None
+    dag: Optional[DAG] = None
         The DAG the task should execute.
     image: Optional[str] = None
         The image to use in the execution of the function.
@@ -165,6 +169,8 @@ class Task(IO):
         start time. This duration also includes time in which the node spends in Pending state.
     metrics: Optional[Union[Metric, List[Metric], Metrics]] = None
         Any built-in/custom Prometheus metrics to track.
+    sidecars: Optional[List[Sidecar]] = None
+        List of sidecars to create for the main pods of the container that runs the task.
 
     Notes
     -----
@@ -179,9 +185,11 @@ class Task(IO):
         source: Optional[Union[Callable, str]] = None,
         with_param: Optional[Any] = None,
         with_sequence: Optional[Sequence] = None,
-        inputs: Optional[List[Union[Parameter, Artifact]]] = None,
+        inputs: Optional[
+            Union[List[Union[Parameter, Artifact]], List[Union[Parameter, Artifact, Dict[str, Any]]], Dict[str, Any]]
+        ] = None,
         outputs: Optional[List[Union[Parameter, Artifact]]] = None,
-        dag: Optional["DAG"] = None,
+        dag: Optional[DAG] = None,
         image: Optional[str] = None,
         image_pull_policy: Optional[ImagePullPolicy] = None,
         daemon: bool = False,
@@ -205,6 +213,7 @@ class Task(IO):
         active_deadline_seconds: Optional[int] = None,
         timeout: Optional[str] = None,
         metrics: Optional[Union[Metric, List[Metric], Metrics]] = None,
+        sidecars: Optional[List[Sidecar]] = None,
     ):
         if dag and source:
             raise ValueError("Cannot use both `dag` and `source`")
@@ -217,7 +226,7 @@ class Task(IO):
         self.source = source
         self.memoize = memoize
         self.volumes = volumes or []
-        self.inputs = inputs or []
+        self.inputs = [] if inputs is None else self._parse_inputs(inputs)
         self.outputs = outputs or []
         self.env = env or []
         self.with_param = with_param
@@ -241,7 +250,8 @@ class Task(IO):
                     "`Optional[Union[Metric, List[Metric], Metrics]]`"
                 )
 
-        self.image = image or get_global_task_image()
+        self.sidecars = sidecars
+        self.image = image or GlobalConfig.image
         self.image_pull_policy = image_pull_policy
         self.daemon = daemon
         self.command = command
@@ -264,16 +274,78 @@ class Task(IO):
         self.when: Optional[str] = None
 
         self.validate()
+
+        # here we cast for otherwise `mypy` complains that Hera adds an incompatible type with a dictionary, which is
+        # an acceptable type for the `inputs` field upon `init`
+        self.inputs = cast(List[Union[Parameter, Artifact]], self.inputs)
         self.inputs += self._deduce_input_params()
 
         if hera.dag_context.is_set():
             hera.dag_context.add_task(self)
 
+        for hook in GlobalConfig.task_post_init_hooks:
+            hook(self)
+
+    @property
+    def id(self) -> str:
+        """Unique ID of container task.
+
+        See Also
+        --------
+        https://argoproj.github.io/argo-workflows/variables/#dag-templates
+        """
+        return f"{{{{tasks.{self.name}.id}}}}"
+
     @property
     def ip(self) -> str:
-        """Returns the specifications for the IP property of the task"""
+        """IP address of the daemon container task.
+
+        See Also
+        --------
+        https://argoproj.github.io/argo-workflows/variables/#dag-templates
+        """
         return f"{{{{tasks.{self.name}.ip}}}}"
 
+    @property
+    def status(self) -> str:
+        """Phase status of the task.
+
+        See Also
+        --------
+        https://argoproj.github.io/argo-workflows/variables/#dag-templates
+        """
+        return f"{{{{tasks.{self.name}.status}}}}"
+
+    @property
+    def exit_code(self) -> str:
+        """Exit code of script or container task.
+
+        See Also
+        --------
+        https://argoproj.github.io/argo-workflows/variables/#dag-templates
+        """
+        return f"{{{{tasks.{self.name}.exitCode}}}}"
+
+    @property
+    def started_at(self) -> str:
+        """Time-stamp when the task started.
+
+        See Also
+        --------
+        https://argoproj.github.io/argo-workflows/variables/#dag-templates
+        """
+        return f"{{{{tasks.{self.name}.startedAt}}}}"
+
+    @property
+    def finished_at(self) -> str:
+        """Time-stamp when the task finished.
+
+        See Also
+        --------
+        https://argoproj.github.io/argo-workflows/variables/#dag-templates
+        """
+        return f"{{{{tasks.{self.name}.finishedAt}}}}"
+
     def _get_dependency_tasks(self) -> List[str]:
         """Extract task names from `depends` string"""
         if self.depends is None:
@@ -391,10 +463,29 @@ class Task(IO):
         """Execute `other` when this task errors."""
         return self.next(other, on=TaskResult.Errored)
 
-    def on_exit(self, other: "Task") -> "Task":
+    def on_exit(self, other: Union["Task", DAG]) -> "Task":
         """Execute `other` on completion (exit) of this Task."""
-        self.exit_task = other.name
-        other.is_exit_task = True
+        # in instances when `other` contains a DAG it is the DAG that needs a template to be
+        # created. Therefore, this "resets" the `other` to be the DAG that needs to be used
+        if isinstance(other, Task) and other.dag is not None:
+            raise ValueError(
+                "Provided `Task` contains a `DAG` set. Only `Task`s with `source` are supported or pure `DAG`s. "
+                "Try passing in `Task.dag` or set `source` on `Task` if you have a single task to run on exit."
+            )
+
+        if isinstance(other, Task):
+            self.exit_task = other.name
+            other.is_exit_task = True
+        elif isinstance(other, DAG):
+            # If the exit task is a DAG, we need to propagate the DAG and its
+            # templates by instantiating a task within the current context.
+            # The name will never be used; it's only present because the
+            # field is mandatory.
+            t = Task("temp-name-for-hera-exit-dag", dag=other)
+            t.is_exit_task = True
+            self.exit_task = other.name
+        else:
+            raise ValueError(f"Unrecognized exit type {type(other)}, supported types are `Task` and `DAG`")
         return self
 
     def on_other_result(self, other: "Task", value: str, operator: Operator = Operator.Equals) -> "Task":
@@ -694,6 +785,7 @@ class Task(IO):
 
         if self.dag:
             if self.dag.inputs:
+                self.dag.inputs = cast(List[Union[Parameter, Artifact]], self.dag.inputs)
                 if len(self.dag.inputs) == 1:
                     deduced_params.append(Parameter(name=self.dag.inputs[0].name, value="{{item}}"))
                 else:
@@ -923,7 +1015,7 @@ class Task(IO):
         else:
             setattr(template, "container", self._build_container())
 
-        affinity = self.affinity._build() if self.affinity else None
+        affinity = self.affinity.build() if self.affinity else None
         if affinity is not None:
             setattr(template, "affinity", affinity)
 
@@ -942,6 +1034,9 @@ class Task(IO):
         if self.metrics is not None:
             setattr(template, "metrics", self.metrics.build())
 
+        if self.sidecars is not None:
+            setattr(template, "sidecars", [sc.build() for sc in self.sidecars])
+
         return template
 
     def _build_tolerations(self) -> List[ArgoToleration]:
